@@ -15,10 +15,12 @@ VM_OPTS="\
 	-Djava.library.path=/usr/lib:/usr/local/lib \
 	"
 
-RELEX_OPTS="\
-	-Drelex.algpath=data/relex-semantic-algs.txt \
-	-Dwordnet.configfile=data/wordnet/file_properties.xml \
-	"
+# By default these resources are read from the jar file or from 
+# the default paths (the same defined below)
+#RELEX_OPTS="\
+#	-Drelex.algpath=data/relex-semantic-algs.txt \
+#	-Dwordnet.configfile=data/wordnet/file_properties.xml \
+#	"
 
 CLASSPATH="-classpath \
 bin:\
@@ -26,6 +28,7 @@ bin:\
 /usr/local/share/java/opennlp-tools-1.2.0.jar:\
 /usr/local/share/java/maxent-2.4.0.jar:\
 /usr/local/share/java/trove.jar:\
+/usr/local/share/java/link-grammar-4.3.4.jar:\
 /usr/share/java/commons-logging.jar:\
 /usr/share/java/gnu-getopt.jar:\
 /usr/share/java/xercesImpl.jar:\
