@@ -1,4 +1,3 @@
-package relex;
 /*
  * Copyright 2008 Novamente LLC
  *
@@ -15,26 +14,35 @@ package relex;
  * limitations under the License.
  */
 
-import java.util.Iterator;
-import java.util.ArrayList;
-import java.util.LinkedHashSet;
+package relex;
 
+import java.util.ArrayList;
+
+import relex.feature.Atom;
 import relex.feature.FeatureNode;
 import relex.feature.LinkableView;
+import relex.stats.SimpleTruthValue;
+import relex.tree.PhraseTree;
 
 /**
- * A ParsedSentence object stores all of the syntactic and semantic information
- * about a sentence parse. The data in the Object is gradually built up by
- * RelationExtractor
+ * A ParsedSentence object stores all of the syntactic and semantic
+ * information about a sentence parse. The data in the Object is
+ * gradually built up by RelationExtractor.
  *
- * ParsedSentence contains: 1. a FeatureNode with metaData about the parse (i.e.,
- * the number of conjunctions) 2. a ArrayList of FeatureNodes (leafConstituents)
- * representing each word in the sentence. -- the parse data can be found by
- * checking the links in these words. 3. Strings representing the original
- * sentence, and representations of its parses 4. Sets of relations, with the
- * semantic data from the sentence.
+ * ParsedSentence contains:
+ * 1. A FeatureNode with metaData about the parse (i.e., the number
+ *    of conjunctions)
+ * 2. An ArrayList of FeatureNodes (leafConstituents) representing each
+ *    word in the sentence. -- the parse data can be found by checking
+ *    the links in these words.
+ * 3. Strings representing the original sentence, and representations
+ *    of its parses
+ * 4. Sets of relations, with the semantic data from the sentence.
+ * 5. A TruthValue (inherited from Atom) that ranks the relative
+ *    likelihood of this parse of being a correct (meaningful) parse
+ *    of the sentence.
  */
-public class ParsedSentence
+public class ParsedSentence extends Atom
 {
 	// String containing the original sentence
 	private String original;
@@ -61,7 +69,8 @@ public class ParsedSentence
 	/* -------------------------------------------------------------------- */
 	/* Constructors, and setters/getters for private members. */
 	// Constructor.
-	public ParsedSentence(String originalString) {
+	public ParsedSentence(String originalString)
+	{
 		original = originalString;
 		linkString = null;
 		errorString = "";
@@ -129,7 +138,8 @@ public class ParsedSentence
 	 * Shows the full feature structure of the parse as it can be found by
 	 * tracing links from the left-most word. Islands will be missed
 	 */
-	public String fullParseString() {
+	public String fullParseString()
+	{
 		if (getLeft() != null)
 			return getLeft().toString();
 		return "";
@@ -168,38 +178,75 @@ public class ParsedSentence
 
 	/* ---------------------------------------------------------------- */
 	/**
-	 * Returns an Iterator over ALL the FeatureNodes in the parse.
-	 * That is, not only are nodes representing the constituents
-	 * returned, but also all their sub-FeatureNodes representing
-	 * links, semantic info, etc.
-	 *
-	 * @return an Iterator over ALL the FeatureNodes in the parse.
+	 * @return the FeatureNode representing the left-most word in the sentence.
 	 */
-	public Iterator<FeatureNode> iteratorFromLeft()
+	public FeatureNode getLeft()
 	{
-		return _iteratorFromLeft(getLeft(), new LinkedHashSet<FeatureNode>()).iterator();
-	}
-
-	private LinkedHashSet<FeatureNode> 
-	_iteratorFromLeft(FeatureNode f, LinkedHashSet<FeatureNode> alreadyVisited) 
-	{
-		if (alreadyVisited.contains(f))
-			return alreadyVisited;
-		alreadyVisited.add(f);
-		if (f.isValued())
-			return alreadyVisited;
-		Iterator<String> i = f.getFeatureNames().iterator();
-		while (i.hasNext())
-			_iteratorFromLeft(f.get(i.next()), alreadyVisited);
-		return alreadyVisited;
+		return leafConstituents.get(0);
 	}
 
 	/**
-	 * @return the FeatureNode representing the left-most word in the sentence.
+	 * @return the phrase tree associated with this parse
 	 */
-	public FeatureNode getLeft() {
-		return leafConstituents.get(0);
-	} // end getLeft
+	public PhraseTree getPhraseTree()
+	{
+		return new PhraseTree(getLeft());
+	}
+
+	/* ---------------------------------------------------------------- */
+
+	public int getAndCost()
+	{
+		return getMeta("and_cost");
+	}
+
+	public int getDisjunctCost()
+	{
+		return getMeta("disjunct_cost");
+	}
+
+	public int getLinkCost()
+	{
+		return getMeta("link_cost");
+	}
+
+	public int getNumSkippedWords()
+	{
+		return getMeta("num_skipped_words");
+	}
+
+	private int getMeta(String str)
+	{
+		FeatureNode fn = metaData.get(str);
+		if (fn == null) return -1;
+		String val = fn.getValue();
+		return Integer.parseInt(val);
+	}
+
+	/**
+	 * Perform a crude parse-ranking based on Link-grammar output.
+	 * The ranking will be stored as the "confidence" of the 
+	 * TruthValue associated with this parse.
+	 */
+	public void simpleRankParse()
+	{
+		SimpleTruthValue stv = new SimpleTruthValue();
+		truth_value = stv;
+		stv.setMean(1.0);  // 1.0 == true -- this is a parse.
+
+		// The weights used here are rather ad-hoc; but the
+		// basic idea is that we want to penalize skipped words
+		// strongly, but disjunct costs not as much. Low link
+		// costs are the tiebreaker.
+		double weight = 0.4 * getNumSkippedWords();
+		weight += 0.2 * getDisjunctCost();
+		weight += 0.06 * getAndCost();
+		weight += 0.012 * getLinkCost();
+
+		weight = Math.exp(-weight);
+
+		stv.setConfidence(weight);
+	}
 
 } // end ParsedSentence
 
