@@ -16,7 +16,6 @@
 package relex;
 
 import java.io.BufferedReader;
-import java.io.File;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.util.ArrayList;
@@ -26,21 +25,31 @@ import java.util.Map;
 import relex.algs.SentenceAlgorithmApplier;
 import relex.anaphora.Antecedents;
 import relex.anaphora.Hobbs;
+import relex.chunk.ChunkRanker;
+import relex.chunk.LexChunk;
+import relex.chunk.LexicalChunker;
+import relex.chunk.PatternChunker;
+import relex.chunk.PhraseChunker;
+import relex.chunk.RelationChunker;
+import relex.concurrent.RelexContext;
 import relex.corpus.DocSplitter;
+import relex.corpus.DocSplitterFactory;
 import relex.corpus.GateEntityMaintainer;
 import relex.entity.EntityInfo;
 import relex.entity.EntityMaintainer;
 import relex.feature.LinkView;
 import relex.frame.Frame;
+import relex.morphy.Morphy;
+import relex.morphy.MorphyFactory;
 import relex.output.OpenCogXML;
 import relex.output.ParseView;
 import relex.output.RawView;
 import relex.output.SimpleView;
 import relex.parser.LinkParser;
 import relex.parser.LinkParserClient;
-import relex.parser.LinkParserJNIClient;
 import relex.parser.LinkParserJNINewClient;
 import relex.parser.LinkParserSocketClient;
+import relex.tree.PhraseMarkup;
 
 /**
  * The RelationExtractor class provides the central processing
@@ -57,100 +66,41 @@ import relex.parser.LinkParserSocketClient;
 public class RelationExtractor
 {
 	public static final int DEFAULT_MAX_PARSES = 100;
+	public static final int DEFAULT_MAX_SENTENCE_LENGTH = 1024;
 	public static final int DEFAULT_MAX_PARSE_SECONDS = 30;
 	public static final int DEFAULT_MAX_PARSE_COST = 1000;
 	public static final String DEFAULT_ALGS_FILE = "./data/relex-semantic-algs.txt";
 
+	/** The LinkParserClient to be used - this class isn't thread safe! */
+	private RelexContext context; 
+	
 	/** Syntax processing */
-	private LinkParser parser = null;
+	private LinkParser parser;
 
 	/** Semantic processing */
-	private SentenceAlgorithmApplier sentenceAlgorithmApplier = null;
-	private PhraseMarkup phraseMarkup = null;
+	private SentenceAlgorithmApplier sentenceAlgorithmApplier;
+	
+	/** Penn tree-bank style phrase structure markup. */
+	private PhraseMarkup phraseMarkup;
 
 	/** Anaphora resolution */
-	public Antecedents antecedents = null;
-	private Hobbs hobbs = null;
-
-	/** Single-threaded-ness hack */
-	private static RelationExtractor singleton = null;
-	
-	public static RelationExtractor getSingletonInstance()
-	{
-		return singleton;
-	}
+	public Antecedents antecedents;
+	private Hobbs hobbs;
 
 	/* ---------------------------------------------------------- */
 	/* Constructors, etc. */
-	public static void init(File algsFile, boolean useSocket)
-	{
-		if (singleton != null) {
-			throw new RuntimeException("RelationExtractor already initialized.");
-		}
-		singleton = new RelationExtractor(algsFile, useSocket, true);
-	}
 
 	public RelationExtractor(boolean useSocket)
 	{
-		// algs file path can be specified with the -Drelex.algpath= flag
-		String algsFileName = System.getProperty("relex.algpath");
-		if ((algsFileName == null) || (algsFileName.length() == 0))
-		{
-			algsFileName = DEFAULT_ALGS_FILE;
-		}
-
-		File algsFile = new File(algsFileName);
-		if ( (algsFile == null) || (!algsFile.canRead()) )
-		{
-			System.err.println("Error reading semantic algorithms file " + algsFile);
-			return;
-		}
-		_newRelex(algsFile, useSocket, true);
-	}
-
-	private RelationExtractor(File algsFile,
-	                          boolean useSocket,
-	                          boolean useNewJNI)
-	{
-		_newRelex(algsFile, useSocket, useNewJNI);
-	}
-
-	private void _newRelex(File algsFile,
-	                  boolean useSocket,
-	                  boolean useNewJNI)
-	{
-		LinkParser p = null;
-		if (!LinkParser.isSingletonCreated())
-		{
-			LinkParserClient lpc = null;
-			if (useSocket) {
-				lpc = LinkParserSocketClient.getSingletonInstance();
-			} else if (useNewJNI) {
-				lpc = LinkParserJNINewClient.getSingletonInstance();
-			} else {
-				lpc = LinkParserJNIClient.getSingletonInstance();
-			}
-			p = LinkParser.createSingletonInstance(lpc);
-		}
-		else
-		{
-			p = LinkParser.getSingletonInstance();
-		}
-		SentenceAlgorithmApplier saa = new SentenceAlgorithmApplier();
-		saa.read(algsFile);
-
-		_newRelex(p,saa);
-	}
-	
-	public RelationExtractor(LinkParser p, SentenceAlgorithmApplier saa)
-	{
-		_newRelex(p,saa);
-	}
-
-	private void _newRelex(LinkParser p, SentenceAlgorithmApplier saa)
-	{
-		sentenceAlgorithmApplier = saa;
-		parser = p;
+		parser = new LinkParser();
+		
+		LinkParserClient lpc = (useSocket) ? new LinkParserSocketClient() : LinkParserJNINewClient.getSingletonInstance();
+		lpc.init();
+		Morphy morphy = MorphyFactory.getImplementation(MorphyFactory.DEFAULT_SINGLE_THREAD_IMPLEMENTATION);
+		context = new RelexContext(lpc, morphy);
+		
+		sentenceAlgorithmApplier = new SentenceAlgorithmApplier();
+		
 		setMaxParses(DEFAULT_MAX_PARSES);
 		setMaxParseSeconds(DEFAULT_MAX_PARSE_SECONDS);
 		setMaxCost(DEFAULT_MAX_PARSE_COST);
@@ -168,19 +118,19 @@ public class RelationExtractor
 	 * but only this many are returned.
 	 */
 	public void setMaxParses(int maxParses) {
-		parser.setMaxParses(maxParses);
+		context.getLinkParserClient().setMaxParses(maxParses);
 	}
 
 	public void setMaxCost(int maxCost) {
-		parser.setMaxCost(maxCost);
+		context.getLinkParserClient().setMaxCost(maxCost);
 	}
 
 	public void setAllowSkippedWords(boolean allow) {
-		parser.setAllowSkippedWords(allow);
+		context.getLinkParserClient().setAllowSkippedWords(allow);
 	}
 
 	public void setMaxParseSeconds(int maxParseSeconds) {
-		parser.setMaxParseSeconds(maxParseSeconds);
+		context.getLinkParserClient().setMaxParseSeconds(maxParseSeconds);
 	}
 
 	/* ---------------------------------------------------------- */
@@ -213,18 +163,16 @@ public class RelationExtractor
 		                               new ArrayList<EntityInfo>());
 		}
 
-		ArrayList<ParsedSentence> currentParses = 
-		               parseSentence(sentence, entityMaintainer);
-		RelexInfo ri = new RelexInfo(sentence, currentParses);
+		RelexInfo ri = parseSentence(sentence, entityMaintainer);
 
-		for (ParsedSentence parse : currentParses)
+		for (ParsedSentence parse : ri.parsedSentences)
 		{
 			// Markup feature node graph with entity info,
 			// so that the relex algs (next step) can see them.
 			entityMaintainer.prepareSentence(parse.getLeft());
 
 			// The actual relation extraction is done here.
-			sentenceAlgorithmApplier.applyAlgs(parse);
+			sentenceAlgorithmApplier.applyAlgs(parse, context);
 
 			// Strip out the entity markup, so that when the 
 			// sentence is printed, we don't print gunk.
@@ -232,6 +180,9 @@ public class RelationExtractor
 
 			// Also do a Penn tree-bank style phrase structure markup.
 			phraseMarkup.markup(parse);
+
+			// Assign a simple parse-ranking score, based on LinkGrammar data.
+			parse.simpleRankParse();
 		}
 
 		// Perform anaphora resolution
@@ -245,21 +196,23 @@ public class RelationExtractor
 	 * currentParses is filled with the ParsedSentences Uses an optional
 	 * EntityMaintainer to work on a converted sentence.
 	 */
-	private ArrayList<ParsedSentence> 
+	private RelexInfo
 	parseSentence(String sentence, EntityMaintainer entityMaintainer)
 	{
 		if (entityMaintainer != null) {
 			sentence = entityMaintainer.getConvertedSentence();
 		}
 		if (sentence == null) return null;
+
 		ArrayList<ParsedSentence> parses = null;
-		if (sentence.length() < 1024) {
-			parses = parser.parse(sentence);
+		if (sentence.length() < DEFAULT_MAX_SENTENCE_LENGTH) {
+			parses = parser.parse(sentence, context.getLinkParserClient());
 		} else {
 			System.err.println("Sentence too long!: " + sentence);
 			parses = new ArrayList<ParsedSentence>();
 		}
-		return parses;
+		RelexInfo ri = new RelexInfo(sentence, parses);
+		return ri;
 	}
 
 	/* ---------------------------------------------------------- */
@@ -274,6 +227,17 @@ public class RelationExtractor
 		System.out.println(msg + elapsed + " millseconds");
 	}
 
+	/* --------------------------------------------------------- */
+
+	private static void prt_chunks(ArrayList<LexChunk> chunks)
+	{
+		for (LexChunk ch : chunks)
+		{
+			System.out.println(ch.toString());
+		}
+		System.out.println("\n======\n");
+	}
+
 	/* ---------------------------------------------------------- */
 	/**
 	 * Main entry point
@@ -286,14 +250,17 @@ public class RelationExtractor
 			" [-g (use GATE entity detector)]" +
 			" [-h (show this help)]" +
 			" [-l (show parse links)]" +
+			" [-m (show parse metadata)]" +
 			" [-n parse-number]" +
 			" [-o (show opencog XML output)]" +
+			" [--pa (show phrase-based lexical chunks)]" +
+			" [--pb (show pattern-based lexical chunks)]" +
+			" [--pc (show relational lexical chunks)]" +
 			" [-r (show raw output)]" +
 			" [-s Sentence (in quotes)]" +
 			" [-t (show parse tree)]" +
 			" [-v verbose]" +
 			" [-x (show cerego XML output)]" +
-			" [--maxParses N]" +
 			" [--maxParseSeconds N]";
 		HashSet<String> flags = new HashSet<String>();
 		flags.add("-c");
@@ -301,7 +268,11 @@ public class RelationExtractor
 		flags.add("-g");
 		flags.add("-h");
 		flags.add("-l");
+		flags.add("-m");
 		flags.add("-o");
+		flags.add("--pa");
+		flags.add("--pb");
+		flags.add("--pc");
 		flags.add("-r");
 		flags.add("-t");
 		flags.add("-v");
@@ -309,27 +280,22 @@ public class RelationExtractor
 		HashSet<String> opts = new HashSet<String>();
 		opts.add("-n");
 		opts.add("-s");
-		opts.add("--maxParses");
 		opts.add("--maxParseSeconds");
 		Map<String,String> commandMap = CommandLineArgParser.parse(args, opts, flags);
 
 		String sentence = null;
-		Integer parseNum = null;
-		Integer maxParses = null;
-		Integer maxParseSeconds = null;
+		int maxParses = 10;
+		int maxParseSeconds = 60;
 
 		// Check for optional command line arguments.
 		try
 		{
-			parseNum = commandMap.get("-n") != null ? 
+			maxParses = commandMap.get("-n") != null ? 
 				Integer.parseInt(commandMap.get("-n").toString()) : 1;
 
 			sentence = commandMap.get("-s") != null ? 
 				commandMap.get("-s").toString() : null;
 
-			maxParses = commandMap.get("--maxParses") != null ?
-				Integer.parseInt(commandMap.get("--maxParses").toString()) : 3;
-
 			maxParseSeconds = commandMap.get("--maxParseSeconds") != null ?
 				Integer.parseInt(commandMap.get("--maxParseSeconds").toString()) : 60;
 		}
@@ -367,12 +333,12 @@ public class RelationExtractor
 
 		// If sentence is not passed at command line, read from standard input:
 		BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
-		DocSplitter ds = new DocSplitter();
+		DocSplitter ds = DocSplitterFactory.create();
 
 		ParseView ceregoView = new ParseView();
 		OpenCogXML opencog = new OpenCogXML();
 		Frame frame = new Frame();
-		
+
 		while(true)
 		{
 			// If no sentence specified on the command line 
@@ -413,15 +379,21 @@ public class RelationExtractor
 					em = gem.process(sentence);
 					re.reportTime("Gate processing: ");
 				}
-	
+
 				re.starttime = System.currentTimeMillis();
 				RelexInfo ri = re.processSentence(sentence,em);
 				re.reportTime("Relex processing: ");
 	
+				int np = ri.parsedSentences.size();
+				if (np > maxParses) np = maxParses;
+				ChunkRanker ranker = new ChunkRanker(np);
+
 				// Print output
 				int numParses = 0;
 				for (ParsedSentence parse: ri.parsedSentences)
 				{
+					ChunkRanker rollup = new ChunkRanker(1);
+
 					System.out.println(sentence);
 					System.out.println("\n====\n");
 					System.out.println("Parse " + (numParses+1) + 
@@ -443,12 +415,58 @@ public class RelationExtractor
 					    (commandMap.get("-o") == null))
 						System.out.println("\n" + parse.getLinkString());
 
+					if (commandMap.get("-m") != null)
+					{
+						System.out.println(parse.getMetaData().toString() + "\n");
+					}
+
+					// Print simple parse ranking
+					Double confidence = parse.getTruthValue().getConfidence();
+					String pconfidence = confidence.toString().substring(0,6);
+					System.out.println("Parse confidence: " + pconfidence);
+					System.out.println(
+						"cost vector = (UNUSED=" + parse.getNumSkippedWords() +
+						" DIS=" + parse.getDisjunctCost() +
+						" AND=" + parse.getAndCost() +
+						" LEN=" + parse.getLinkCost() + ")");
+
+					// Verbose graph.
 					if (commandMap.get("-v") != null)
 						System.out.println("\n" + parse.getLeft().toString(LinkView.getFilter()));
+
 					System.out.println("\n======\n");
 					System.out.println(SimpleView.printRelations(parse.getLeft()));
 					System.out.println("\n======\n");
 	
+					if (commandMap.get("--pa") != null)
+					{
+						System.out.println("Phrase tree-based lexical chunks:");
+						LexicalChunker chunker = new PhraseChunker();
+						chunker.findChunks(parse);
+						prt_chunks(chunker.getChunks());
+						rollup.add(chunker.getChunks(), parse.getTruthValue());
+					}
+					if (commandMap.get("--pb") != null)
+					{
+						System.out.println("Pattern-matching lexical chunks:");
+						LexicalChunker chunker = new PatternChunker();
+						chunker.findChunks(parse);
+						prt_chunks(chunker.getChunks());
+						rollup.add(chunker.getChunks(), parse.getTruthValue());
+					}
+					if (commandMap.get("--pc") != null)
+					{
+						System.out.println("Relation-based lexical chunks:");
+						LexicalChunker chunker = new RelationChunker();
+						chunker.findChunks(parse);
+						prt_chunks(chunker.getChunks());
+						rollup.add(chunker.getChunks(), parse.getTruthValue());
+					}
+
+					// Now roll them up. Need to do this in two stages
+					// to get the probability calculations correct.
+					ranker.add(rollup.getChunks(), parse.getTruthValue());
+
 					if (commandMap.get("-c") != null)
 					{
 						ceregoView.setParse(parse);
@@ -488,10 +506,16 @@ public class RelationExtractor
 						System.out.println(new String (eot));
 						System.out.println("data\n<!-- ======\n");
 					}
-	
-					if (++numParses >= parseNum) break;
+
+					if (++numParses >= maxParses) break;
 				}
-				System.out.println("\nAntecedent candidates:\n" + re.antecedents.toString());
+				if (0 < ranker.getChunks().size())
+				{
+					System.out.println("\nLexical Chunks:\n" +
+					             ranker.toString());
+				}
+				System.out.println("\nAntecedent candidates:\n"
+				                   + re.antecedents.toString());
 	
 				sentence = ds.getNextSentence();
 			}
